Package: MatrixQCvis
Type: Package
Title: Shiny-based interactive data-quality exploration for omics data
Version: 1.20.0
Date: 2025-06-26
Authors@R: c(person("Thomas", "Naake", role = c("aut", "cre"), 
		email = "thomasnaake@googlemail.com",
		comment = c(ORCID = "0000-0001-7917-5580")),
    person("Wolfgang", "Huber", role = c("aut"), 
		email = "wolfgang.huber@embl.de",
		comment = c(ORCID = "0000-0002-0474-2218")))
VignetteBuilder: knitr
Description: 
	Data quality assessment is an integral part of preparatory data analysis 
	to ensure sound biological information retrieval. 
	We present here the MatrixQCvis package, which provides shiny-based 
	interactive visualization of data quality metrics at the per-sample and 
	per-feature level. It is broadly applicable to quantitative omics data types 
	that come in matrix-like format (features x samples). It enables the detection 
	of low-quality samples, drifts, outliers and batch effects in data sets.
	Visualizations include amongst others bar- and violin plots of the (count/intensity) 
	values, mean vs standard deviation plots, MA plots, empirical cumulative 
	distribution function (ECDF) plots, visualizations of the distances 
	between samples, and multiple 
	types of dimension reduction plots. Furthermore, MatrixQCvis allows for 
	differential expression analysis based on the limma (moderated t-tests) and 
	proDA (Wald tests) packages. MatrixQCvis builds upon the popular 
	Bioconductor SummarizedExperiment S4 class and enables thus the facile 
	integration into existing workflows. The package 
	is especially tailored towards metabolomics and proteomics mass spectrometry 
	data, but also allows to assess the data quality of other data types that 
	can be represented in a SummarizedExperiment object.
Depends: R (>= 4.1.0), DT (>= 0.33), SummarizedExperiment (>= 1.20.0),
        plotly (>= 4.9.3), shiny (>= 1.6.0)
Imports: ComplexHeatmap (>= 2.7.9), dplyr (>= 1.0.5), ExperimentHub (>=
        2.6.0), ggplot2 (>= 3.3.3), grDevices (>= 4.1.0), Hmisc (>=
        4.5-0), htmlwidgets (>= 1.5.3), impute (>= 1.65.0), imputeLCMD
        (>= 2.0), limma (>= 3.47.12), MASS (>= 7.3-58.1), methods (>=
        4.1.0), pcaMethods (>= 1.83.0), proDA (>= 1.5.0), rlang (>=
        0.4.10), rmarkdown (>= 2.7), Rtsne (>= 0.15), shinydashboard
        (>= 0.7.1), shinyhelper (>= 0.3.2), shinyjs (>= 2.0.0), stats
        (>= 4.1.0), sva (>= 3.52.0), tibble (>= 3.1.1), tidyr (>=
        1.1.3), umap (>= 0.2.7.0), UpSetR (>= 1.4.0), vsn (>= 3.59.1)
Suggests: BiocGenerics (>= 0.37.4), BiocStyle (>= 2.19.2), hexbin (>=
        1.28.2), httr (>= 1.4.7), jpeg (>= 0.1-10), knitr (>= 1.33),
        statmod (>= 1.5.0), testthat (>= 3.0.2)
biocViews: Visualization, ShinyApps, GUI, QualityControl,
        DimensionReduction, Metabolomics, Proteomics, Transcriptomics
License: GPL-3
Encoding: UTF-8
RoxygenNote: 7.3.1
git_url: https://git.bioconductor.org/packages/MatrixQCvis
git_branch: RELEASE_3_23
git_last_commit: 521dbc7
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 23:05:01 UTC; biocbuild
Author: Thomas Naake [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-7917-5580>),
  Wolfgang Huber [aut] (ORCID: <https://orcid.org/0000-0002-0474-2218>)
Maintainer: Thomas Naake <thomasnaake@googlemail.com>
