### Barplot for number of features

The plot shows the number of missing features 
along the samples. Missing values are defined as `NA` values in `assay(se)`.
The plot will help to identify samples that 
have a high number of missing values that can be excluded from further analysis.

