### Boxplot or violin plots of assay values among the samples

This tab shows the assay values for 

 - raw (**raw**), 
 - normalized (**normalized**),
 - normalized+batch corrected (**batch corrected**), 
 - normalized+batch corrected+transformed (**transformed**), 
 - normalized+batch corrected+transformed+imputed (**imputed**) 
 
assay values. The **imputed** values are only displayed if there are missing 
values (`NA`s) in the **raw** data set. The different methods for 
normalization, transformation, batch correction and imputation are specified 
in the side panel. 

For visualization purposes only, the assay values for the raw and normalized 
data sets can be `log` transformed 
(**Display log values? (only for 'raw' and 'normalized')**).

The type of visualization (boplot or violin plot) can be specified 
by selecting `boxplot` or `violin` in the radio button panel. 