### Distance matrices between samples

On the left, the panel depicts heatmaps of distances between samples for 
the data sets of
raw (**raw**), normalized (**normalized**), normalized+transformed 
(**transformed**), normalized+transformed+batch corrected 
(**batch corrected**), and normalized+transformed+imputed (**imputed**).
The annotation of the heatmaps can be adjusted by the drop-down
menu **annotation**.

On the right panel the sum of distances to other samples is depicted. 