### Distance matrices between samples

On the left, the panel depicts heatmaps of distances between samples for 
the data sets of

 - raw (**raw**), 
 - normalized (**normalized**), 
 - normalized+batch corrected (**batch corrected**),
 - normalized+batch corrected+transformed (**transformed**), and 
 - normalized+batch corrected+transformed+imputed (**imputed**).
 
The **imputed** data set is only displayed if there are 
missing values (`NA`s) in the **raw** data set. 
The annotation of the heatmaps can be adjusted by the drop-down
menu **annotation**.

On the right panel the sum of distances to other samples is depicted. 