### Mosaic plot
The plot displays information from two categorical/qualitative variables. 
Use the select inputs (**Categorical 1** and 
**Categorical variable 2**) to specify the variables on the x-axis and
y-axis. The area of the files is proportional to the number 
of observations within that category (also displayed in %).