### t-distributed stochastic neighbor embedding (tSNE)

The panel depicts a plot of tSNE. The parameters **Perplexity**, 
**Number of iterations**, **Number of retained dimensions in initial PCA**,
and **Output dimensionality** required for the tSNE algorithm can be
set with the sliders. For the parameter
**Number of retained dimensions in initial PCA** the panel 
*Principal components* can be employed that shows on the left panel a 
Scree plot of the data set and permuted values and corresponding p-values
from the permutation. For computational reasons, the data set will
be restricted to the top 5000 varying features based on their 
standard variation (only if there are more than 5000 features 
in the data set, otherwise all features are taken). 

The different dimensions of the transformed data can be displayed by changing 
the values in the drop-down menus for the x-axis and y-axis (either two or 
three dimensions according to the **Output dimensionality**).  