### Principal coordinates analysis (PCoA)

The panel depicts a plot of PCoA (= multidimensional scaling). Different distance 
measures can be used to calculate the distances between the samples 
(euclidean, maximum, manhattan, canberra, minkowski). 
The different axes of the transformed data can be displayed by changing the
values in the drop-down menus for the x-axis and y-axis. 