### Barplot for number of features

The plot shows the number of measued features along the samples. Measured 
values are defined as non-`NA` values in `assay(se)`.
The plot will help to identify samples that 
have a high number of missing values that can be excluded from further analysis.
