### Histogram for number of features along a variable
The plot can be read accordingly to the one in the tab *Histogram features*, 
but, it is segregated for the stratified variable (e.g. it shows the 
distribution of missing values among the sample types). 

The variable for stratification can be specified by the drop-down menu
**Variable for stratification**.
