### Boxplot or violin plots of values among the samples

This tab shows the assay values for raw (**raw**), normalized 
(**normalized**), normalized+transformed (**transformed**), and
normalized+transformed+batch corrected (**batch corrected**)
assay values. 
The different methods for normalization, transformation, and batch 
correction are specified in the side panel. 

For visualization purposes only, the assay
values for the raw and normalized data sets can be `log2` transformed
(**Display log2 values? (only for 'raw' and 'normalized')**).

The type of visualization (boplot or violin plot) can be specified 
by selecting `boxplot` or `violin` in the radio button panel. 