### Mean-sd plots

The panel shows the three mean-sd (standard deviation) plots for  
normalized+transformed (**transformed**), 
normalized+transformed+batch corrected (**batch corrected**), and 
normalized+transformed+batch corrected+imputed (**imputed**) assay values.
The sd and mean are calculated feature-wise from the values of the
respective data set. The plot allows to visualize if there is a dependence
of the sd on the mean. The red line depicts the running median estimator 
(window-width 10%). In case of sd-mean independence, the running median
should be approximately horizontal. 
