### Uniform Manifold Approximation and Projection (UMAP)

The panel depicts a plot of UMAP. The parameters **Minimum distance**, 
**Number of neighbors**, and **Spread**, required for the UMAP algorithm can be
set with the sliders. 

The different dimensions of the transformed data can be displayed by changing 
the values in the drop-down menus for the x-axis and y-axis.  