### Histogram for number of features

The plot shows how often a feature was missing in a certain number of samples.  

Examples: 

1. in the case of one sample (x-axis), the y-axis will denote the number of 
features for which only one feature was missing. 

2. in the case of the number of total samples (x-axis), the y-axis will denote
the number of features with completely missing observations (i.e. the number of 
features for which no feature was quantified in all samples). 

The binwidth is determined by the slider 
input (**Binwidth (Measured Values)** or **Binwidth (Missing Values)**). 

