### Intersection of sets among a variable (UpSet plot)

The plot shows the interaction of sets between different variables depending 
on their presence. The dots in the UpSet plot specify if the 
criteria for presence are fulfilled:
Presence is defined by a feature being measured in at least one sample of a
set. 
 
In each column the intersections are displayed together with the number 
of features regarding the type of intersection. The boxplots in the rows 
display the number of present features per set. 

All sets present in the data set are displayed.  Depending on the data set, 
however, not all intersections sets are displayed 
(see the help page for the `upset` function from the `UpSetR` package). 
